/*
 *           OMEGA SECURITY SERVICES
 * 	     (c) 2008 Omega Dev Team
 *
 *
 *                 $Id: g_opm.c 1549 2009-12-13 06:54:06Z twitch $
 */


#include "stdinc.h"
#include "server.h"

#include "opm.h"
#include "opm_error.h"
#include "opm_types.h"
#include "compat.h"

#define NAME "g_opm"
#define AUTHOR "Omega Team"
#define VERSION "$Id: g_opm.c 1549 2009-12-13 06:54:06Z twitch $"

#define ARRAY_SIZEOF(x) (sizeof(x) / sizeof((x)[0]))

#if 0
	OPM_T *scanner;
#endif

pthread_t complete = 0;

void opm_entry(void *);
void opm_exit (void *);


void open_proxy(OPM_T *, OPM_REMOTE_T *, int, void *);
void negotiation_failed(OPM_T *, OPM_REMOTE_T *, int, void *);
void opm_timeout(OPM_T *, OPM_REMOTE_T *, int, void *);
void end(OPM_T *, OPM_REMOTE_T *, int, void *);
void handle_error(OPM_T *, OPM_REMOTE_T *, int, void *);

static void event_opmcon(int, User *);


int opm_scan_count = 0;
int opm_thread_bail = 0;

dlink_list proxy;
pthread_cond_t opm_cond =  PTHREAD_COND_INITIALIZER;

Thread* opm_thread;


unsigned short http_ports[] = {
    8000, 8080, 3128, 80
};

unsigned short wingate_ports[] = {
    23
};

unsigned short router_ports[] = {
    23
};

unsigned short socks4_ports[] = {
    1080
};

unsigned short socks5_ports[] = {
    1080
};

unsigned short httppost_ports[] = {
    80, 8090, 3128
};


int gopm_load();
void gopm_unload();


MODHEADER(NAME, VERSION, AUTHOR, MAKE_ABI(0,5,2), gopm_load, gopm_unload);


int gopm_load()
{

	int status, ch;
	//if we dont have pthreads bail before we do anything. 
	#ifndef HAVE_PTHREAD
		throwModErr("Module Error, Threading is disabled or not available.","");
	#endif

	AddEvent ("USERCONN", event_opmcon);

	#ifdef HAVE_PTHREAD
		if (opm_thread = spawn_thread(NULL, opm_entry, opm_exit))
			return MOD_CONT;
		else
			return MOD_STOP;
	#endif

	return MOD_CONT;
	
}




void gopm_unload()
{
	thread_cancle(opm_thread);
	DelEvent("USERCON",event_opmcon);
	return;
}




static void event_opmcon(int ac, User *u)
{

	dlink_node *dl;
	char *ip;

	if ((u->service) || (!u))
		return;

	if (sync_state != 3)
		return;

	sendto_one_notice (s_Guardian, u, "Your connection is being scanned for open proxies");

	dl = dlink_create();
	dlink_add_tail (opm_remote_create(u->ip), dl, &proxy);
	opm_scan_count++;
	
	thread_signal(&opm_cond);
	
	return;
}

void opm_exit (void *args)
{
        dlink_node *dl,*tdl;
        OPM_REMOTE_T *remote;

        DLINK_FOREACH_SAFE(dl,tdl,proxy.head)
        {
                remote = dl->data;
                dl = dlink_find_delete(remote,&proxy);
                dlink_free(dl);
                opm_remote_free(remote);
                opm_scan_count--;
        }

        return;
}

void opm_entry(void *args)
{
    OPM_REMOTE_T *remote;
    OPM_T *scanner;

    OPM_ERR_T err;
    int fdlimit = 1024;
    int max_read = 4096;
    int scantimeout  = 10;
    int port = 6667;

    unsigned int i, s;
    int status, ch;

    char *user_ip;

    dlink_node *dl, *tdl;

    time_t opm_scan_timeout = 0;

    int err_code = 0;


    scanner = opm_create();

    /* Setup callbacks */
    opm_callback(scanner, OPM_CALLBACK_OPENPROXY, &open_proxy, NULL);
    opm_callback(scanner, OPM_CALLBACK_NEGFAIL, &negotiation_failed, NULL);
    opm_callback(scanner, OPM_CALLBACK_TIMEOUT, &opm_timeout, NULL);
    opm_callback(scanner, OPM_CALLBACK_END, &end, NULL);
    opm_callback(scanner, OPM_CALLBACK_ERROR, &handle_error, NULL);


    /* Setup the scanner configuration */
    opm_config(scanner, OPM_CONFIG_FD_LIMIT, &fdlimit);
    opm_config(scanner, OPM_CONFIG_SCAN_IP, CfgSettings.local_ip);
    opm_config(scanner, OPM_CONFIG_SCAN_PORT, &port);
    opm_config(scanner, OPM_CONFIG_TIMEOUT, &scantimeout);
    opm_config(scanner, OPM_CONFIG_MAX_READ, &max_read);

    /* Setup the protocol configuration */
    for (s = ARRAY_SIZEOF(http_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_HTTP, http_ports[i]);


    for (s = ARRAY_SIZEOF(wingate_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_WINGATE, wingate_ports[i]);


    for (s = ARRAY_SIZEOF(router_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_ROUTER, router_ports[i]);


    for (s = ARRAY_SIZEOF(socks4_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_SOCKS4, socks4_ports[i]);


    for (s = ARRAY_SIZEOF(socks5_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_SOCKS5, socks5_ports[i]);


    for (s = ARRAY_SIZEOF(httppost_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_HTTPPOST, httppost_ports[i]);


    //below this line is a cluster fuck ;/
    while ((sync_state != SHUTDOWN) || (opm_thread_bail != 1))
    {
            if (opm_scan_count <= 0)
            {
                    sleep(1);
                    continue;
            }
			
			mutex_state(opm_thread, THREAD_MUTEX_LOCK);
			
			printf("Thread Active\n");
			
            DLINK_FOREACH_SAFE(dl,tdl,proxy.head)
            {
				
                remote = dl->data;

                err_code = opm_scan(scanner, remote);

                if (err_code != OPM_SUCCESS)
                    break;

                opm_scan_timeout = time(NULL) + 180;

                while (opm_active(scanner) > 0)
                {
                    	if (time(NULL) > opm_scan_timeout)
                        	break;

                    	opm_cycle(scanner);
                    	sleep(1);
                }

                dl = dlink_find_delete(remote,&proxy);
                dlink_free(dl);
                opm_remote_free(remote);
                opm_scan_count--;					
			}
			mutex_state(opm_thread, THREAD_MUTEX_UNLOCK);
			thread_wait(opm_thread, &opm_cond);
			
    }

    opm_free(scanner);
    dest_thread();
	return;
}


void open_proxy(OPM_T *scanner, OPM_REMOTE_T *remote, int notused, void *data)
{
   USE_VAR(notused);
   USE_VAR(data);

   sendto_logchan("\002Proxy:\002 Found proxy on %s:%d", remote->ip,
         remote->port);
    ircd_xline("Z", s_Guardian->nick, remote->ip, "Detected possible proxy from %s:%d.", remote->ip,remote->port);
    return; //meh dont continue keep matching.
}


void negotiation_failed(OPM_T *scanner, OPM_REMOTE_T *remote, int notused,
      void *data)
{
    USE_VAR(scanner);
    USE_VAR(notused);
    USE_VAR(data);

    return;
}

void opm_timeout(OPM_T *scanner, OPM_REMOTE_T *remote, int notused, void *data)
{

    USE_VAR(scanner);
    USE_VAR(notused);
    USE_VAR(data);

	return;
}



void end(OPM_T *scanner, OPM_REMOTE_T *remote, int notused, void *data)
{

   USE_VAR(scanner);
   USE_VAR(notused);
   USE_VAR(data);


   complete = pthread_self();
   return; //dont destroy the thread let it handle that as needed. :)
}

void handle_error(OPM_T *scanner, OPM_REMOTE_T *remote, int err, void *data)
{

   USE_VAR(scanner);
   USE_VAR(data);

   switch(err)
   {
      case OPM_ERR_MAX_READ:
            break;
      case OPM_ERR_BIND:
            break;
      case OPM_ERR_NOFD:
            sendto_logchan("Unable to allocate file descriptor for %s:%d",
               remote->ip, remote->port);
            break;
      default:
          sendto_logchan("Unknown error on %s:%d, err = %d", remote->ip,
               remote->port, err);
   }
}

