#include "stdinc.h"
#include "server.h"

char *LogType[] = {
	"Info",     //0
	"Debug",    //1
	"Debug 2",  //2
	"Error",    //3
	"Critical", //4
	"Event",    //5
	"Link",     //6
	"LogUser",  //7
	"LogChan",  //8
	"LogIO",    //9
	"Module",   //10
	"Debug3",   //11
	"Backtrace", //12
	0
};

static unsigned long curday = 0;

/*************************************************/
/**
  * Init Function - Called when server starts.
  */



/*************************************************/
void open_log (void)
{
    char    LogName[512];
    char    timestamp[32];

    time_t      t;
    struct tm   *tmp;

    time (&t);
    tmp = localtime (&t);

    if (logfd > 0)
    {
        if (tmp->tm_mday == curday)
            return;
        else
            fclose (logfd);
    }

    snprintf (LogName, sizeof(LogName), "%s%s%s-%d%d%d",
            DPATH, LPATH, (CfgSettings.network)? CfgSettings.network : "security",
            (tmp->tm_year + 1900), (tmp->tm_mon + 1), tmp->tm_mday);

    curday  = tmp->tm_mday;

    logfd   = fopen (LogName, "a");

    if (!logfd)
        fprintf (stderr, "Warning: Could not open logfile (%s): %s\n", LogName, strerror(errno));

    return;
}
/*************************************************/

void close_logs	(void)
{

	if (logfd)
	{
		    fflush(logfd);
			fclose(logfd);
	}
	return;
}


/*************************************************/

void alog(int TYPE, char *fmt, ...)
{

    char logstr[1025];
	va_list		args;

    open_log ();

	//check for bad fd
    if (!logfd)
        return;

	va_start(args,fmt);
	vsnprintf(logstr,1024,fmt,args);

	switch (TYPE)
	{
            case  DEBUG:
                    if (!debug)
                            return;
                    break;
            case DEBUG_2:
                    if (debug < 2)
                            return;
                    break;
            case DEBUG3:
                    if (debug < 3)
                            return;
                    break;

	}
	if (nofork && debug)
            printf("[%s] %s\n",LogType[TYPE], logstr);

    fprintf(logfd,"[%lu] %s\n",ServerTime,logstr);
    fflush(logfd);
	va_end(args);
	return;
}

/***********************************/

void close_log(void)
{
    if (logfd)
	    fclose (logfd);
}

/***********************************/
void sendto_logchan (char *fmt, ...)
{
    va_list     args;
    char        tmp[1024];

    if ((uselogchan != 1) || (sync_state != RUNNING))
		return;

    va_start (args, fmt);
    vsnprintf (tmp, 1024, fmt, args);
    va_end (args);

    if (!s_Guardian)
        return;

    if ((sync_state == 3) && logchan)
	 send_line (":%s PRIVMSG %s :%s", (IRCd.ts6)? s_Guardian->uid : s_Guardian->nick, logchan, tmp);

    return;
}


/************************************/

void sendto_console(char *fmt, ...)
{
     va_list     args;
     char        tmp[1025];

     va_start (args, fmt);
     vsnprintf (tmp, 1024, fmt, args);

     fprintf(stderr,"%s\r\n", tmp);

    va_end(args);
    return;

}

/************************************************/
 /**
 *  Log - This function logs to our log file, as
 *        well as sends to logchan... This doesnt
 *        do any implicit checks, and will rely on
 *        the user to know what he/she is doing.
 *
 *  @param TYPE - The log level/type
 *  @param fmt  - Formatted VA_ARG strings, or
 *                standard parameters.
 *  @return void
 */


void Log(char *fmt, ...)
{
     va_list     args;

     char tmp[1024];

     va_start (args, fmt);
     vsnprintf (tmp, 1024, fmt, args);

    sendto_logchan(tmp); //send to our logchan

    alog(0,tmp);      //then alog


    va_end(args);
    return;

}

/************************************/

