#include "stdinc.h"
#include "server.h"

static void showHelp ();

extern int uselogchan;

/***************************************/

void Run        (void);
void DaemonSeed (void);
void Banner     (void);

void Load_Dbs (void);

ServCmds* servcmds_list = NULL;
UserCmds* usercmds_list = NULL;

ModuleMessage* modmsg_list = NULL;
Module* module_list        = NULL;

static void AtExit(void);
void do_restart(void);

extern void psr_HandleEvent(Socket* s,int et);

//copy of our arguments and enviorment
char **my_env,**my_argv;

/***************************************/

int main(int argc, char **argv, char **env)
{
	int c;

	my_argv	= argv;

    uselogchan  = 1;
    starttime   = time(NULL);
    sync_state = 0; //set our state to not connected.

	SetTime();

        sprintf (CfgSettings.conf_name, "%s/%s", EPATH, "security.cfg");

        my_env = env;

        while ((c = getopt (argc, argv, "rshbvnd:c:l:")) != -1)
        {
            switch (c)
            {
                case 'h':
					showHelp ();
                    break;
                case 'v':
                        printf("Version: \033[1;31m%s %s-%s\033[0m\n",PACKAGE_NAME,PACKAGE_VERSION,VERSION_STRING);
                        exit(0);
                case 's':
                        nomodules = 1;
                        break;
                case 'd':
						if (optarg)
						{
							debug	= atoi(optarg);

							if ((debug < 1) || (debug > 3))
							{
								fprintf (stderr, "Invalid value for -d (%d)\n", debug);
								exit (0);
							}
						}
                        break;
                case 'n':
                        nofork = 1;
                        break;
                case 'c':
						if (optarg) {
	                        if ((optarg[strlen(optarg) - 4] == '.') && optarg[strlen(optarg) - 3] == 'c')
	                            sprintf (CfgSettings.conf_name, "%s/%s", EPATH, optarg);
							else
								fprintf(stderr,"Invalid config file specified with -c option skipping.\n");
						}
                        break;
                case 'l':
                        if (optarg)
                            memcpy(logchan,optarg,sizeof(logchan));
                        break;
            }
        }


        //check to see if were running under root and bail
        if ((getuid() == 0) && (getgid() == 0))
        {
            printf("Error: You are currently trying to run this program as root\n");
            printf("This program does not require root privledges ...\n");
            exit(0);
        }

        if (chdir (DPATH))
        {
              fprintf (stderr, "Unable to chdir to %s: %s\n", DPATH, strerror(errno));
              exit (EXIT_FAILURE);
        }


        init_lists ();

        InitDefaults ();
        init_config();
        ConfLoad ();
        VerifyConf ();

        DaemonSeed();

        atexit(AtExit);

        Run();

        //we returned from Run() Some how :/ i shit you not.. we need to exit ;( *tear*
            Exit(0);
        return 0;
}


/******************************************/

void Banner(void)
{
    printf("\n");
    printf("Omega Security Services Daemon\n");
    printf(" Build: \033[1;31m%s %s\033[0m\n",PACKAGE_VERSION,VERSION_STRING);
    printf(" Compiled: \033[1;31m%s\033[0m\n",COMPILE_DATE);
    printf(" Process ID: \033[1;31m%lu\033[0m\n",(unsigned long) getpid());
    printf("\r\n");
    printf(" Developers: \033[1;32m Twitch Jer\033[0m\n");
    printf("\r\n");
}



/******************************************/

void DaemonSeed(void)
{
        FILE *pidfd;
	int our_pid;


        if (nofork != 1)
        {


            //check to see if were currently running :)
            if ((our_pid = check_for_pid()) > 0) {
			printf("Omega is currently running [Pid: %d]\n", our_pid);
			exit(1);
	    }

            //here we go forking our process
            if ((pid = fork ()) < 0) {
                 printf("ERROR: Unable to fork process\n");
                 exit(1);
            }  else if (pid > 0) {
                 sleep(2);
                 exit(0);
            }

            setsid ();


            pidfd = fopen(CfgSettings.pidfile,"w+");
            fprintf(pidfd,"%lu",(unsigned long) getpid());
            fflush(pidfd);
            fclose(pidfd);

        }
}


/***********************************/
/**
 * Move this to its own function
 *
 * return 0 if there is no pid or -1 if there is no file else
 * return the current active pid.
 */

int check_for_pid() {
  	    FILE *pidfd;
	    int our_pid;
	    int value;
       	char tmp[20];

 	    if ((pidfd = fopen(CfgSettings.pidfile,"r")))
        {

			memset(tmp,0,sizeof(tmp));

			fread(tmp,1,sizeof(tmp),pidfd);
		   	fclose(pidfd);
		  	our_pid = atoi(tmp);
		
			value = kill((pid_t) our_pid, 0);
		
			switch (value)
			{
				case ESRCH:
					//destroy the pid file
					unlink (CfgSettings.pidfile);
				 	return 0;
				case 0:
					return our_pid;
		
			}
	    } 
	    return -1; //no pid file.
}

/******************************************/
//there has to be a better way to do this :/
void Run(void)
{
    SetSig       ();
    Banner       ();


#ifdef HAVE_SETPROCTITLE
    setproctitle(CfgSettings.servername);
#endif

    open_log  ();
    init_uid ();
	init_access_flags();
	
    AddTimedEvent ("House Keeping", ev_housekeeping, 900);
    AddTimedEvent ("Uplink Ping", ev_ping, 120);
    AddEvent ("BURST", burst_local_users);

    if ((Me = new_serv(NULL, CfgSettings.servername)))
    {
        strlcpy (Me->sid, CfgSettings.sid, sizeof(Me->sid));
        Me->eos = 1; //mark us as already EOS'ed
    }
    else
    {
		fprintf (stderr, "Unable to create our server entry\n");
		exit (0);
    }

    /*
     * Initialize our core for running
     */

	 
    protocol_init ();

    servsock    	 = add_new_socket ((void*) psr_HandleEvent, NULL);
    servsock->flags	|= SOCK_UPLINK;
	strlcpy (servsock->name, "Uplink", sizeof(servsock->name));

    psr_init      ();

    /*
     * Load modules, clients, and db's
     */

    if (!load_protocol())
        return;

    if (!load_core())
        return;

    load_modules();
    load_client_modules();
	
	Rehash(-1); //this is a hax for now but reload the config files.
	
    introduce_users ();

    /*
     * Connect to the IRCd if we dont connect Add an event to connect...
     * also OneTimeAround will know if we are threaded or not and handle
     * accordingly, also if we are threaded this will launch our threads
     * and individual socket engines per fd... so we dont get lagged.
     */

    printf ("\r\n");
    printf ("Connecting to \033[1;32m%s\033[0m port \033[1;32m%d\033[0m\033[0m\n", CfgSettings.uplink, CfgSettings.port);
    if (Connect (servsock, CfgSettings.uplink, CfgSettings.vhost, CfgSettings.port))
        ircd_connect ();
    else
        AddTimedEvent ("Connect Uplink", ev_connectuplink, 60);

    /*
     * Okay begin
     */

    OneTimeAround   ();

    return;
}

/******************************************/

void OneTimeAround()
{
       while (1) 
       {
	      /*
	       * run these in the core its self...
	       * i hated the fact the select engine 
	       * ran these.
	       */
	      SetTime ();
	      RunTimedEvents ();
	      //runModuleQue();
      
	      //allow for better core control over the socketengine!
	      switch (receive(NULL))
	      {
				default:
					break;
	      }
	}
	return;       // We should only reach this if we are shutting down
}

/******************************************/

void SetTime() {
	time_t t;
       struct tm *tmp;

       time(&t);
       tmp = localtime(&t);

	//record the server time as were passnig through
	ServerTime = (unsigned long) time(NULL);

	//check to see if its a new date :) or
	//if we just start set our last date to
	//our current date :)

	if ((curdate) && (tmp))
	{
		if (curdate->tm_mday != tmp->tm_mday)
			lastdate = curdate;
	}
	curdate = tmp;
 }

/******************************************/
/**
 * Standard function definitions.
 */

int lengh(char *s1) {
        int x;
        if (s1) {
                x=0;
                while (s1[x]) {
                        x++;
                }
        }
        return x;
}

/******************************************/

int compair(char *s1,char *s2) {
    int x;

    assert(s1);
    assert(s2);

     if (!s2 || !s1)
		return 1;

      for (x=1; s1[x] && s2[x]; x++) {

        if (s1[x] != s2[x])
		return 1;
    }

    return 0;
}

/******************************************/
/**
  * AtExit() Called by atexit when program terminates
  * this insures we have proper cleanup :)
  *
  * @return void - must return void
  */

static void AtExit(void)
{

    sync_state = SHUTDOWN;

    uplink_cleanup("Exiting",1);

    unlink (CfgSettings.pidfile);

	socket_cleanup(); //destroy all outgoing socket stuff.
		
    DestAccList();

#ifdef HAVE_PTHREAD
    dest_all_threads();
#endif

    unload_all_modules();

#ifdef USE_DMALLOC
	dmalloc_shutdown();
#endif

	close_log();
    return;
}

/******************************************/
/**
 *  Load_Dbs()
 *
 */

void Load_Dbs(void)
{
	return;
}
/******************************************/
/**
 * Our exit function - since everything is pretty much handled
 * in AtExit we are just calling exit(0) for the time being
 */

void Exit(int status)
{
	exit(status);
}

/******************************************/

void Rehash(int status)
{
	if (status >= 0)
		sendto_logchan("Reloading configuration files");
		
	DestAccList();
	ConfReload();
	VerifyConf();
	refresh_access();
	rehash_client_modules();
	rehash_core_modules();
}



/******************************************/

void do_restart()
{
	uplink_cleanup ("Restarting", 1);

	Event ("RESTART", 0, NULL);
	unlink (CfgSettings.pidfile);

	execv(SPATH, my_argv);

	sleep(1);

	exit(0);
	return;
}






/**
 **
 */

void init_lists	()
{
	userlist.head	= userlist.tail	= NULL;
	servlist.head	= servlist.tail	= NULL;
	events.head		= events.tail   = NULL;
	sockets.head	= sockets.tail	= NULL;
	accesslist.head	= accesslist.tail	= NULL;
	sockevents.head	= sockevents.tail	= NULL;
	access_flags.head = access_flags.tail = NULL;
	
//	module_list.head	= module_list.tail	= NULL;

	connected_sockets.head	= connected_sockets.tail			= NULL;
	
	

	return;
}

/**
 **
 */

static void showHelp ()
{
	printf ("Available Arguments\n");
	printf ("\n\n");

	printf ("Runtime options:\n");
	printf ("-d[1|2|3]    run in debug mode\n");
	printf ("-n           Do not fork into the background.\n");
	printf ("-b           Generate backtraces internally.\n");
	printf ("-r           Reload configuration and modules.\n");
	printf ("-s           Run in skeleton mode, Additional modules disabled.\n");
	printf ("-v           Display version and exit\n");
	printf ("-h           Display this help\n");
	printf ("\n\n");

	printf ("Configuration options:\n");
	printf ("-c [config name]    Use the named configuration.\n");
	printf ("-l [log channel]    Use the specified log channel.\n");

	exit (0);
}

/* vim: set tabstop=4 shiftwidth=4 expandtab */

