/*
 *         	  OMEGA IRC SECURITY SERVICES
 * 	      	    (C) 2008 Omega Dev Team
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *    $Id: module.c 1552 2009-12-14 06:57:11Z twitch $
 */

#include "stdinc.h"
#include "server.h"

extern int modcnt;

char *GetModErr(int);

int module_que_count = 0;

/*****************************************/
/**
 * Return a module error string.
 *
 * @param status Error code we are returning.
 *
 * @note negative value of status is representive of module
 * side errors, these are custom errors that modules
 * must define.
 */


char *GetModErr(int status)
{


if (status <= 0)
	{
		switch (status)
		{
			case MOD_ERR_MSG:
				return (char *) mod_err_msg;
		}
	}


	const char *ModErr[] = {
			"Success, no error.",
			"Module Error, Module Returned STOP.",
			"Module Error, CRITICAL.",
			"Module Error, FATAL.",
			"Module Error, File already loaded.",
			"Module Error, No such file.",
			"Module Error, IO Error.",
			"Module Error, Invalid API.",
			"Module Error, Incorrect ABI version.",
			"Module Error, Allocating Memory.",
			"Module Error, Unable to unload module.",
			"Module Error, Invalid SYNTAX.",
			"Module Warning, Unsafe to unload.",
			"Module Error, Core set in no-module mode.",
			"Module Error, SQLite is not available on this system."
	};

	if (ModErr[status])
		return (char*)ModErr[status];
	else
		return (char*)"Module Warning, No return from MODULE";

}


/*****************************************/
/**
 * load_protocol() - Load our given protocol
 *                   if it fails toss a critical
 *                   error and return
 *  @return bool
 *              true  - Module has loaded successfully
 *              false - Module failed to load see logs
 *
 */


int load_protocol()
{
    int status = 0;
    char *pmod = (char*) malloc(sizeof(char) * 255);

	if (!pmod)
		return 0;

 	sprintf(pmod,"%s%s",PPATH,CfgSettings.protocol);

	if (!strstr(pmod,".so"))
		strcat(pmod,".so");

       status = module_open(pmod, MOD_TYPE_PROTOCOL);

	if ((status!= 0 && status != 12))
	{
       	sendto_console("FATAL: %s \033[1;34m %s \033[0m",CfgSettings.protocol, GetModErr(status));
		alog(4,"Protocol Error: [%s] %s",pmod, GetModErr(status));
		return 0;
	}

    free(pmod);
    return 1;

}

/*****************************************/


int load_core() {
	DIR *dp;
    struct dirent *ep;

	char path[255];

    int status = 0;


	printf("Loading Core ");

	if (!(dp = opendir(CPATH)))
	{
		perror("Unable to open core module directory.");
		return 0;
	}

	while (ep = readdir (dp))
    {
		if (*ep->d_name == '.')
			continue;

            if ((ep->d_name[strlen(ep->d_name) - 3] != '.') ||
					(ep->d_name[strlen(ep->d_name) - 2] != 's')) {
				continue;
			}

		sprintf(path,"%s%s",CPATH,ep->d_name);

		status = module_open(path, MOD_TYPE_CORE);

		if (status != 0)
		{
			printf("\n %s \033[1;34m %s \033[0m \n", ep->d_name, GetModErr(status));
			return 0;
		}
		printf(".");
		memset(path,0,sizeof(path));
	}

    (void) closedir (dp);

	printf(" Done\n\n");
    return 1;
}

/*****************************************/


int load_modules()
{
    int i;
    int status = 0;
    DIR *dp;
    struct dirent *ep;
    char path[255];
    char *end_curfile = NULL; //end of the current file


	if (!nomodules)
	{
		printf("\r\n");
		printf("Loading Modules\n");
		if (!(dp = opendir(MPATH)))
		{
			perror("Error");
			return 0;
		}

		while (ep = readdir (dp))
		{

			if (*ep->d_name == '.')
				continue;

			if ((ep->d_name[strlen(ep->d_name) - 3] != '.') ||
					(ep->d_name[strlen(ep->d_name) - 2] != 's')) {
				continue;
			}

			sprintf(path,"%s/%s",MODULE_DIR,ep->d_name);

			status = module_open(path, MOD_TYPE_STD);

			printf(" * Loading: \033[1;32m %s \033[0m \033[1;34m %s \033[0m \n", ep->d_name,(status != 0)? GetModErr(status) : "");
			memset(path,0,sizeof(path));
		}

		(void) closedir (dp);

		return MOD_ERR_OK;

	} else {

		printf("\r\n");
		printf("Note: Core is currently running in skeleton mode\n");
		printf("      No non-core modules will be loaded\n");
		return MOD_ERR_OK;
	}

	return MOD_ERR_OK;

}



/*****************************************/

int load_client_modules()
{
	int cnt = 0;
	int status = 0;
	char path[255];

	if (nomodules)
		return;

	//first init the guardian moodules
	while (cnt < gmod_cnt)
	{
		sprintf(path,"%s/%s",GMOD_DIR,guardian_modules[cnt]);
		status = module_open(path, MOD_TYPE_CLIENT);
		printf(" * Loading: \033[1;32m %s \033[0m \033[1;34m %s \033[0m \n", guardian_modules[cnt],(status != 0)? GetModErr(status) : "");
		memset(path,0,sizeof(path));
		cnt++;
	}
	return 0;
}


/*****************************************/

/**
  * module_open() - Load our module file symbols
  *
  * @param filename The file we are trying to link
  *              against.
  * @param type  Type of module we are loading.
  *              - 0 MOD_TYPE_UNKNOWN Unknown module type it was either loaded
  *                after or manualy.
  *              - 1 MOD_TYPE_PROTOCOL
  *              - 2 MOD_TYPE_CORE
  *              - 3 MOD_TYPE_STD
  *              - 4 MOD_TYPE_CLIENT
  *
  * @return int
  *  - 0 = MOD_ERR_OK   Loading is successful no error.
  *	 - 1 = MOD_ERR_STOP We recieve a stop either from the module or core.
  *  - 2 = MOD_ERR_CRIT Criticial error happened while loading.
  *  - 3 = MOD_ERR_FATAL Fatal error occured while loading.
  *  - 4 = MOD_ERR_EXISTS Module is already loaded.
  *  - 5 = MOD_ERR_NOFILE No such file/directory
  *  - 6 = MOD_ERR_IO We were unable to read symbols from the module file.
  *  - 7 = MOD_ERR_API No open/closeing functions loaded
  *  - 8 = MOD_ERR_ABI In Omega ABI refers to the BINARY INTERFACE version
  *		               which is used to in dicate wether a module will run or not
  *                    if this is returned the ABI values do not match.
  *  - 9 = MOD_ERR_MEMORY We were unable to allocate memory.
  *  - 10 = MOD_ERR_UNLOAD Module file is unsafe to unload
  *  - 11 = MOD_ERR_SYNTAX Someything funky was passed to us.
  *  - 13 = MOD_ERR_NOMOD Core is running in skeleton no moudles can be loaded
  *
  *
  *  @note we dont wanna exit if a module fails just return fail status, so
  *  the caller can handle errors.
  */


int module_open(char *filename, int type)
{
  	Module* m;
	dlink_node *dl;
	char  tmp[256];


    /**
     * If we have no structure loaded from the module we wont get a new value
     * for status. So with this in mind set this to invalid API so we know
     * this when we return and error off.
     */

	int mod_status = MOD_ERR_API;


    format_filename(tmp,filename);

    /**
    * Now make sure the module exsists, and isnt already loaded.
    */

    if (!module_exists(tmp))
        return MOD_ERR_NOFILE; //no such file

    if (module_find(tmp))
        return MOD_ERR_EXISTS; //module already loaded aka exists in the core


    /**
    * Initialize the structures
    */

    if (!(m = (Module*) malloc(sizeof(Module))))
        return MOD_ERR_MEMORY;

    m->age = time(NULL);
    strlcpy(m->file,tmp,sizeof(m->file) - 1);

    /**
    * Open the module for loading since all
    * our sanaty checks are done.
    */


    if (!(m->handle = dlopen(tmp, RTLD_NOW)))
    {
            module_free(m);
            throwModErr("%s",dlerror());
    }

    /**
    * Okay first step and only load step is to load
    * the ModuleInfo from the modules... This will
    * greately decrease IO as we load symbols.
    */


    if (!(m->mi = (ModuleInfo*) dlsym(m->handle,"ModInfo")))
    {
            module_free(m);
            throwModErr("%s",dlerror());
    }
	
	/* Theoreticly patches should be fixes that dont affect 
	 * any currently made modules... If it does then thats a minor fix...
	 * and ajustments need to be made
	 */
	if ((ABI_VERSION - ABI_PATCH) > m->mi->abi)
	{
		module_free(m);
		return MOD_ERR_ABI;
	}

    /**
    * Now all the symbols that we need are loaded
    * and placed into the module structure run
    * the register function.
    */

    if (m->mi->mod_register) {

            if ((mod_status = m->mi->mod_register()) != 0)
                    return mod_status;

    } else {
            module_free(m);
            return MOD_ERR_API;
    }



    dl = dlink_create();
    dlink_add_tail (m, dl, &modules);

    alog(10,"Loaded Module: %s",m->mi->name);

    return MOD_ERR_OK;
}


/************************************************/


int module_close(char *filename)
{
    char  tmp[256];
    Module* module;
    dlink_node *dl;

    /**
    * Make sure our path is correct and a aboslute path as well.
    */

    format_filename(tmp, filename);

    if (!(module = module_find(tmp)))
            return MOD_ERR_NOFILE;

    if (module->mi->mod_unregister) {
        module->mi->mod_unregister();
    } else {
        throwModErr("Invalid API, no unload function found for [%s] unable to unload",filename);
    }

    alog(10,"Unloading module: %s",filename);

    dlclose(module->handle);
    module_free(module);

    return MOD_ERR_OK;
}


/************************************************/

Module* module_find(char *filename)
{
    dlink_node* dl;
    Module* m;

    char file[256];

    format_filename(file,filename);

    DLINK_FOREACH (dl, modules.head)
    {
        m = dl->data;

        if (strcasecmp(m->file,filename)==0)
                        return m;
    }

    return NULL;
}


/*********************************************************/


void module_free(Module *m)
{

    dlink_node *dl;

	assert (m != NULL);

    if ((dl = dlink_find_delete(m,&modules)))
        dlink_free(dl);

	MyFree(m);

}

/**********************************************************/
/**
 *   ModuleQ - Que module operations so that we dont
 *   unload modules while they are active...
 *
 *   @param modname - The name of the module that we are
 *                    queing up.
 *   @param op      - The action to take when module q
 *                    is ran.
 *
 *   return MOD_ERR_*
 *
 */


int addModuleQue(char *modname, int op)
{

	dlink_node* dl = dlink_create ();
	ModuleQEntry *entry;

	if ((op == 1) && (!(module_find(modname))))
		return MOD_ERR_NOFILE;

	entry = (ModuleQEntry*) malloc(sizeof(ModuleQEntry));

	entry->name = strdup(modname);
	entry->action = op;

	dlink_add_tail(entry,dl,&moduleque);

	alog(DEBUG3,"Adding %s to module que (Action %d)",modname,op);

	return MOD_ERR_OK;
}


int runModuleQue()
{
	dlink_node *dl,*tdl;
	ModuleQEntry *entry;
	int found = 0;
	int status = 0;

	DLINK_FOREACH_SAFE(dl, tdl, moduleque.head)
	{
			entry = dl->data;

			alog(DEBUG3,"Running action %d on module %s",entry->action,entry->name);

			switch (entry->action)
			{
                case 2:
                    break; //gonna be reload.
			    case 1:
                    status = module_close(entry->name);
                    sendto_logchan("\002Module:\002 Unloading module [%s] - %s",
                            entry->name, GetModErr(status));
                    break;
                case 0:
                    status = module_open(entry->name, MOD_TYPE_UNKNOWN);
                    sendto_logchan("\002Module:\002 Loading module [%s] - %s",
                            entry->name, GetModErr(status));
                    break;

			}

		free(entry->name);
		dl = dlink_find_delete(entry,&moduleque);
		dlink_free(dl);
		free(entry);

	}

	return 0;
}


/**********************************************************/

int module_exists (char * fileName)
{
   struct stat buf;
   char file[256];

	int	i;

   format_filename(file,fileName);

   i = stat ( file, &buf );

   memset(file,0,sizeof(file));

   return (i == 0)? 1 : 0;

}

/**********************************************************/


char *find_module_dir(char *module)
{
	char path[255];
	//first look in mpath
	sprintf(path,"%s/%s",MPATH,module);

	if (module_exists(path))
		return MPATH;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",PPATH,module);

	if (module_exists(path))
		return MPATH;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",CPATH,module);

	if (module_exists(path))
		return CPATH;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",GMOD_DIR,module);

	if (module_exists(path))
		return GMOD_DIR;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",CONTRIB_PATH,module);

	if (module_exists(path))
		return CONTRIB_PATH;

	memset(path,0,sizeof(path));

	return MPATH; //return MPATH no matter what as our module_open will just shoot out module doesnt exsists

}

/**************************************************************/


int rehash_client_modules()
{

	Module *m;
	int cnt = 0;
	char path[255];
	int status = 0;

	for (cnt = 0; cnt < gmod_cnt; cnt++)
	{
		memset(path,0,sizeof(path));
		sprintf(path,"%s/%s",GMOD_DIR,guardian_modules[cnt]);

		if ((m =  module_find(path)))
			continue;

		status = module_open(path, MOD_TYPE_CLIENT);
		sendto_logchan("\002Module:\002 Loading module %s - %s",guardian_modules[cnt],GetModErr(status));
	}
	return 0;
}


/*****************************************************************/


int rehash_core_modules()
{

	DIR *dp;
    struct dirent *ep;
	char path[255];
	Module* m;

	int status = 0;

	if (!(dp = opendir(CPATH)))
	{
		sendto_logchan("\002Rehash:\002 Unable to open core module directory [%s].",CPATH);
		return 1;
	}

	while (ep = readdir (dp))
       {
		m = NULL;

		if (*ep->d_name == '.')
			continue;

		sprintf(path,"%s%s",CPATH,ep->d_name);

		if ((m = module_find(path)))
			continue;

		status = module_open(path, MOD_TYPE_CORE);
		sendto_logchan("Module: Loading module %s - %s", ep->d_name, GetModErr(status));

	}

	(void) closedir (dp);



	if (!(dp = opendir(MODULE_DIR)))
	{
		sendto_logchan("\002Rehash:\002 Unable to open core module directory [%s].",MPATH);
		return 1;
	}


	while (ep = readdir (dp))
       {
		m = NULL;

		if (*ep->d_name == '.')
			continue;

            if (ep->d_name[strlen(ep->d_name) - 3] != '.')
                    continue;

            sprintf(path,"%s/%s",MODULE_DIR,ep->d_name);

		if ((m = module_find(path)))
			continue;

		status = module_open(path,MOD_TYPE_STD);
		sendto_logchan("Module: Loading module %s - %s", ep->d_name, GetModErr(status));

	}

       (void) closedir (dp);

	return 0;

}






/*****************************************************************/

void unload_all_modules()
{

    dlink_node *dl,*tdl;
    Module *module;

	DLINK_FOREACH_SAFE (dl,tdl, modules.head)
	{
            module = dl->data;
            module_close(module->file);
	}

}




