#include "stdinc.h"
#include "server.h"

static char	sock_getbyte (Socket *);
static int	sock_readline (Socket *);

static void	sendq_add	(Socket *, char *, int, int);
static int	sendq_flush (Socket *);

int		maxfds;

#ifdef USE_KQUEUE
static struct kevent    *kqlst;
static int				kqmax;
static int				kqoff;
static int				kq;

static void socket_exit  (void *);

void init_kqueue (void);

void init_kqueue (void)
{
	kq	= kqueue ();
	kqmax	= getdtablesize ();
	kqlst	= malloc (sizeof (struct kevent) * kqmax);
	return;
}
#endif



//these are safe to declare as definitions are
//are safe for machines with/o pthreading - Twitch
Thread* SocketThread;

static void socket_entry(void* args)
{
	receive((Socket *) args);

	dest_thread();

	return;
}


static void socket_exit(void* args)
{
	dlink_node *dl;

	Socket* s = (Socket*) args;

	if (s->flags & SOCK_UPLINK)
	{

		uplink_cleanup(NULL,0);
		return;

	} else {
		dl = dlink_find_delete (s, &sockets);
		dlink_free (dl);
		free (s);
	}

	return;

}

Socket *add_new_socket(void (*read)(Socket*, int),void (*write)(void*))
{
        Socket  	*s;
        dlink_node	*dl;

        s = (Socket *) malloc(sizeof(Socket));

        memset (s->name, '\0', sizeof(s->name));

        s->sd           = -1;
        s->fullLines    =  0;

        s->flags		= 0x0;

        s->user			              = NULL;

        s->mbuffer.head = s->mbuffer.tail         = NULL;
        s->sendq.head	= s->sendq.tail		= NULL;

        s->read_buffer[0]	= '\0';

        dl	= dlink_create ();

        dlink_add_tail	(s, dl, &sockets);

        memcounts.socketcnt++;
        memcounts.socketsize	+= sizeof(s);

        add_sock_handlers(s, read,write);

        return s;
}

Socket * new_socket ()
{
         Socket  	*s;
	 dlink_node	*dl;

         s = (Socket *) malloc(sizeof(Socket));

	 memset (s->name, '\0', sizeof(s->name));

         s->sd           = -1;
         s->fullLines    =  0;

	 s->flags		= 0x0;

	 s->user			              = NULL;

         s->mbuffer.head = s->mbuffer.tail         = NULL;
	 s->sendq.head	= s->sendq.tail		= NULL;

	 s->read_buffer[0]	= '\0';

	 dl	= dlink_create ();

	 dlink_add_tail	(s, dl, &sockets);

	 memcounts.socketcnt++;
	 memcounts.socketsize	+= sizeof(s);

     return s;
}

Socket *find_socket (char *name)
{
	Socket		*s;
	dlink_node	*dl;

	DLINK_FOREACH (dl, sockets.head)
	{
		s	= (Socket *) dl->data;

		if (s->name[0] != '\0')
			if ((strcasecmp (s->name, name)) == 0)
				return s;
	}

	return NULL;
}


/************************************************/

#ifdef HAVE_GETADDRINFO
struct addrinfo	* gethostinfo (char const *host, int port);

int Connect(Socket* s, char *server, char *vhost, int port)
{
	struct addrinfo	*hostres, *res;
	struct addrinfo *bindres = NULL;
	int optval, flags;

        s->sd   = -1;

	hostres		= gethostinfo (server, port);

	if (hostres == NULL)
	{
		fprintf (stderr, "Lookup failure for hostname %s\n", server);
        return -1;
	}

	if (*vhost != '\0')
	{
		bindres	= gethostinfo (vhost, 0);

		if (bindres == NULL)
		{
			fprintf (stdout, "Connection Failed! (Lookup failure for vhost %s)\n", vhost);
            return -1;
		}
	}

	for (res = hostres; res != NULL; res = res->ai_next)
	{
		s->sd	= socket (res->ai_family, res->ai_socktype, res->ai_protocol);

		if (s->sd < 0)
			continue;

		optval	= 1;

		setsockopt (s->sd, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, sizeof(optval));

		if (!(s->flags & SOCK_UPLINK))
		{

			flags	= fcntl (s->sd, F_GETFL, 0);
			flags	|= O_NONBLOCK;
			(void) fcntl (s->sd, F_SETFL, flags);
		}

		if (bindres != NULL)
		{
			if (bind (s->sd, bindres->ai_addr, bindres->ai_addrlen) < 0)
			{
				fprintf (stdout, "Connect Failed! (Unable to bind to %s)\n", vhost);
				s->sd	= -1;
				return -1;
			}
		}

		if ((connect (s->sd, res->ai_addr, res->ai_addrlen)) != 0)
		{
			if (!(flags & O_NONBLOCK))
			{
				fprintf (stdout, "Connect Failed! (%s)\n", strerror(errno));
				s->sd	= -1;
            			return -1;
			}
		}
		if (s->sd > maxfds)
			maxfds	= s->sd;

		if (!(s->flags & SOCK_UPLINK))
			s->flags |= SOCK_WRITE;
		else
			s->flags |= SOCK_READ;

		break;
	}

	freeaddrinfo (hostres);


	if (bindres != NULL)
		freeaddrinfo (bindres);

    return s->sd;
}
#else
void Connect(Socket* s, char *server, char *vhost, int port)
{
	struct hostent *he = NULL;
	struct sockaddr_in my_sin;
	struct in_addr * in;


	if((s->sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("Unable to bind Socket\n");
		exit(1);
	}



	if (*vhost != '\0')
	{
		memset ((void *) &my_sin, '\0', sizeof(my_sin));
		my_sin.sin_family	= AF_INET;

		if ((he = gethostbyname (vhost)) == NULL)
		{
			printf (" Failed! (Invalid Address)\n");
			exit (1);
		}

		in	= (struct in_addr *) (he->h_addr_list[0]);
		my_sin.sin_addr.s_addr	= in->s_addr;
		my_sin.sin_port = 0;

		if (bind (s->sd, (struct sockaddr *) &my_sin, sizeof(my_sin)) < 0)
		{
			printf ("Connection Failed! (Unable to bind to %s)\n", vhost);
			exit (1);
		}
	}

	he = gethostbyname(CfgSettings.uplink);
	if (!he)
	{
		printf ("Invalid Address\n");
		exit (1);
	}

	my_sin.sin_family = AF_INET;
	my_sin.sin_port = htons(port);	/* port, htons converts integers to network short */
	my_sin.sin_addr = *((struct in_addr *) he->h_addr);
	memset(my_sin.sin_zero, 0, sizeof(my_sin.sin_zero)); /* sin_zero is 8 bytes used for padding */

	if(connect(s->sd, (struct sockaddr *)&my_sin, sizeof(struct sockaddr)) == -1)
	{
		perror("Connect\n");
		exit(1);
	}

	return s->sd;


}
#endif


/************************************************/


struct in_addr *GetHost(char *address)
{
	return NULL;
}


/************************************************/

int receive(Socket *s)
{
	dlink_node	*dl, *tdl;
	int		rc;
	int 		modque = 0;
	struct timeval  timeout;

#ifdef USE_KQUEUE
	int				kq, nev, i;
	struct kevent	chlist[2];
	struct kevent	evlist[2];

	if ((kq = kqueue()) == -1)
	{
		perror("kqueue");
		exit (EXIT_FAILURE);
	}
#endif

#ifndef USE_KQUEUE
		FD_ZERO(&wfdset);
		FD_ZERO(&rfdset);
		FD_ZERO(&errfdset);


		DLINK_FOREACH_SAFE (dl, tdl, sockets.head)
		{
			s	= dl->data;


			// Clean out dead sockets
			if ((s->flags & SOCK_DEAD) || (s->sd == -1))
					continue;
			else if (s->flags & SOCK_CONNECTING) {
				if (s->flags & SOCK_DCCOUT)
					FD_SET (s->sd, &wfdset);
				else if (s->flags & SOCK_DCCIN)
					FD_SET (s->sd, &rfdset);
				else if (s->flags & SOCK_UPLINK)
					FD_SET (s->sd, &wfdset);
				else if (s->flags & SOCK_READ)
					FD_SET (s->sd, &rfdset);
				else
					FD_SET (s->sd, &wfdset);
			} else
				FD_SET (s->sd, &rfdset);
		}


		timeout.tv_sec = 0;
		timeout.tv_usec = 250 * 1000;


		rc = select (maxfds + 1, &rfdset, &wfdset, NULL, &timeout);

		if (rc > 0) // This indicates we have fd's ready for read/write
		{
				DLINK_FOREACH_SAFE (dl, tdl, sockets.head)
				{
					s	= (Socket *) dl->data;

                    if ((!s) || !(s->sd > 0))
                        continue;

					if (FD_ISSET (s->sd, &rfdset))
					{
						if (s->flags & SOCK_CONNECTING)
						{
							s->read_callback(s,EVENT_READ);
							continue;
						}

						sock_readline (s);
						s->read_callback(s,EVENT_READ);

					}

					if (FD_ISSET (s->sd, &wfdset))
					{
					    if (s->write_callback)
						s->write_callback(s);
					}
				}
			return 0;
		}
		else
		{
			if (rc == 0)    // Again 0 is okay
				return 0;

			else if ((rc == -1) && (errno == EINTR)) // Interrupted system call, we got a signal, simply restart the select loop
				return 0;

			else
			{
				/* Select() error, handle */
				printf ("[%d]: %s\n", errno, strerror(errno));
				exit (-1);
			}
		}
#else

		DLINK_FOREACH_SAFE (dl, tdl, sockets.head)
		{
			s	= (Socket *) dl->data;

			if (s->sd == -1)
				continue;

			if (s->flags & SOCK_CONNECTING)
			{
				if (s->flags & SOCK_DCCOUT)
					;
			}
		}

		DLINK_FOREACH (dl, connected_sockets.head)
		{
			s	= dl->data;

			if (s->sd != -1)
				EV_SET(&chlist[0], s->sd, EVFILT_READ, EV_ADD | EV_ENABLE, 0, 0, 0);
			else if (s->dead)
				EV_SET(&chlist[0], s->sd, 0, EV_DELETE, 0, 0, 0);
		}

		for (;;)
		{
			nev	= kevent (kq, chlist, 1, evlist, 1, NULL);

			if (nev < 0)
			{
				perror ("kevent()");
				exit (EXIT_FAILURE);
			}

			if (nev > 0)
			{
				if (evlist[0].flags & EV_EOF)
					exit (EXIT_FAILURE);
				break;
			}
		}

		if (nev == 0)
			continue;

		for (i = 0; i < nev; i++)
		{
			if (evlist[i].flags & EV_ERROR)
			{
				fprintf (stderr, "EV_ERROR: %s (%d)\n", strerror(evlist[i].data), evlist[i].ident);
				exit (EXIT_FAILURE);
			}

			DLINK_FOREACH (dl, connected_sockets.head)
			{
				s	= dl->data;

				if (evlist[i].ident == s->sd)
				{
					MyRead (s);
					HandleSockEvent (EVENT_READ);
				}
			}
		}
#endif /* USE_KQUEUE */
	/* If we reach here something has broke us out of the select loop */
	return 0;
}

/************************************************/

int send_line(char *fmt,...)
{
	char	tmp[1024];

	int n;
	va_list args;

	if (!servsock)
		return 0;

	if ((!servsock->sd) || (servsock->sd <= 0))
		return 0;

	if (!fmt)
		return -1;

	memset(tmp,0,sizeof(tmp));

	va_start(args,fmt);
	vsnprintf(tmp,1024,fmt,args);
	va_end (args);

	alog(2,"SEND: %s", tmp);

	if (!strstr(tmp,"\r")) { strcat(tmp,"\r"); }
	if (!strstr(tmp,"\n")) { strcat(tmp,"\n"); }


	if (strlen(tmp) > 512)
		return -1;

#if 0
	/**
	 ** This is our example of throttling
	 ** Allows 10 lines every 2 seconds
	 */

	thisline	= time(NULL);

	if (!lastline)
		lastline	= thisline;

	if ((thisline - lastline) < 2)
	{
		if (lines > 10)
		{
			sendq_add (sock, tmp, strlen(tmp), 0);
			return;
		}
		else
			lines++;
	}
	else
	{
		lastline = thisline;
		lines = 0;
	}

#endif


	/* This should prevent sending from blocking */
	n	= sendq_flush (servsock);

	if (n == -1)
	{
		if (errno != EAGAIN)
		{
			uplink_cleanup (NULL, 0);
            AddTimedEvent ("Connect Uplink", ev_connectuplink, 60);
			/* Catch errors and close the uplink socket */
			return 1;
		}
		else
		{
			sendq_add (servsock, tmp, strlen(tmp), 0);
			return 0;
		}
	}
	else if (n == 0)
	{
		sendq_add (servsock, tmp, strlen(tmp), 0);
		return 0;
	}

	if ((n = write (servsock->sd, tmp, strlen(tmp))) == -1)
	{
		if (errno != EAGAIN)
		{
			uplink_cleanup (NULL, 0);
            AddTimedEvent ("Connect Uplink", ev_connectuplink, 60);
			/* Catch errors and close the uplink socket */
			return 1;
		}
		else
		{
			sendq_add (servsock, tmp, strlen(tmp), 0);
			return 0;
		}
	}
       sentbytes += n;

	if (n != strlen(tmp))
		sendq_add (servsock, tmp, strlen(tmp), n);

	return 0;
}

static char sock_getbyte (Socket *s)
{
	static char		message[MAXLEN + 1];
	static char		*c = message;
	char			rc;
	int				n;
	static int		data_in_buffer = 0;

	if (data_in_buffer == 0)
	{
		n = read (s->sd, message, MAXLEN);

		if (n <= 0)
		{
			if (s->flags & SOCK_UPLINK)
			{
				uplink_cleanup (NULL, 0);
				AddTimedEvent ("Connect Uplink", ev_connectuplink, 60);
				return '\0';
			}
			else
			{
				shutdown (s->sd, SHUT_RDWR);
				close (s->sd);

				s->flags	|= SOCK_DEAD;

				return '\0';
			}
		}

		message[n] = '\0';
		c = message;
		data_in_buffer = 1;
	}

	if (*c == '\0')
	{
		data_in_buffer = 0;
		message[0] = '\0';
		return *c;
	}

	rc = *c;
	c++;
	return rc;
}

static int sock_readline (Socket *s)
{
	char	message[MAXLEN + 1];
	int				i;
	char			c;
	MessageBuffer	*m;
	dlink_node		*dl;

	i = 0;

	if (s->read_buffer[0] != '\0')
	{
		strlcpy (message, s->read_buffer, (MAXLEN + 1));
		i = strlen(message);
		message[i] = '\0';
		s->read_buffer[0] = '\0';
	}

	while (TRUE)
	{
		c = sock_getbyte (s);

		if (s->flags & SOCK_DEAD)
			return 1;

		if (c == '\0')
		{
			if (i > 0)
			{
				message[i] = '\0';
				strlcpy (s->read_buffer, message, MAXLEN + 1);
			}

			break;
		}

		if ((c == '\r') || (c == '\n'))
		{
			if (i > 0)
			{
				message[i] = '\0';

				recvbytes++;

				dl	= dlink_create ();
				m	= (MessageBuffer *) malloc (sizeof(MessageBuffer));

				memset (m->message, '\0', sizeof(m->message));

				memcounts.msgbufcnt++;
				memcounts.msgbufsize	+= strlen(message);

				strlcpy (m->message, message, sizeof(m->message));

				s->fullLines++;
				dlink_add_tail (m, dl, &s->mbuffer);

				i = 0;
			}

			continue;
		}

		message[i] = c;
		i++;
	}

	return 0;
}

/*************************************************/
/**
* Return a pointer to our buffered lines
*/

char *GetLine (Socket *s)
{
    dlink_node      *dl, *tdl;
    char            *line;
    MessageBuffer   *m;

    //dont even try to continue
    if (s->fullLines == 0)
        return NULL;


		/**
		  * block loss prevention -.- some how were spitting out WAY to many blocks here
		  *
		  * ==21769==  49,428 bytes in 96 blocks are definately lost in loss record 40 of 41
		  * ==21769==       at 0x40053C0 malloc (vg_replace_malloc.c:149)
		  * ==21769==       by 0x804D3FF GetLine (socketengine.c:433)
		  *
		  * This occurse on the malloc below  - Twitch
		  */

    line = (char *) malloc(MAXLEN + 1);

    if (!line)
    {
	alog(3,"Unable to allocate memory in %s",__FILE__);
	return NULL;
    }


    DLINK_FOREACH_SAFE (dl, tdl, s->mbuffer.head)
    {
        m   = dl->data;

        if (!m->message)
        {
            s->fullLines    = 0;
            return NULL;
        }

        memcpy (line, m->message, MAXLEN);

        memcounts.msgbufsize  -= strlen(line);
        memcounts.msgbufcnt--;


        dlink_delete (dl, &s->mbuffer);
        dlink_free (dl);

        free(m);

	s->fullLines--;

	return line;
    }

    return NULL;
}

/**
 ** SendQ commands
 */

static void sendq_add (Socket *s, char *buf, int len, int pos)
{
	struct sendq	*sq;
	dlink_node		*dl = dlink_create ();

        sq          = (struct sendq *)malloc(sizeof(struct sendq));

	memcounts.sendqsize	+= sizeof(sq);
	memcounts.sendqcnt++;

	sq->buf		= strdup (buf);
	sq->len		= len - pos;
	sq->pos		= pos;
        sq->s           = s->sd;

	dlink_add_tail (sq, dl, &s->sendq);
}

static int sendq_flush (Socket *s)
{
	struct sendq	*sq;
	int 			n;
	dlink_node		*dl, *tdl;

	DLINK_FOREACH_SAFE (dl, tdl, s->sendq.head)
	{
		sq	= dl->data;

		if (sq->s > 0) //only write if our fd is valid else just purge stuff out
		{
			if ((n = write (sq->s, sq->buf + sq->pos, sq->len)) == -1)
			{
				if (errno != EAGAIN)
					return -1;
				return 0;
			}
		}
        if (n == sq->len)
        {
			memcounts.sendqcnt--;
			memcounts.sendqsize -= sizeof(sq);

			dlink_delete (dl, &s->sendq);
			free (sq->buf);
			free (sq);
			dlink_free (dl);
        }
        else
        {
            sq->pos += n;
            sq->len -= n;
            return 0;
        }
    }

    return 1;
}

#ifdef HAVE_GETADDRINFO
/* Stolen from FreeBSD's whois client, modified for Sentinel by W. Campbell */
struct addrinfo * gethostinfo(char const *host, int port)
{
  struct addrinfo hints, *res;
  int error;
  char portbuf[6];

  memset(&hints, 0, sizeof(hints));
  hints.ai_flags = 0;
  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  snprintf(portbuf, 6, "%d", port);
  error = getaddrinfo(host, portbuf, &hints, &res);
  if (error)
  {

    alog(3, "gethostinfo error: %s: %s", host,
         gai_strerror(error));
    return (NULL);
  }
  return (res);
}
#endif



/********************************************/
/**
 * SendToken() - This function checks wether
 *               or not we have specified
 *               to use tokens if we have
 *               it returns the tokens
 *               if not then it returns reg
 *
 *  @param reg - Non token command. IE NICK
 *  @param tok - Tokenized version of the
 *               command IE &
 *  return char* the correct version
 *               we are using
 *
 */


char *SendToken(char *reg,char *tok)
{
	if (UseTokens)
		return tok;
    else
		return reg;

}

/********************************************/
/**
  *  send_cmd()  Sends a command from source
  *                          to the server.
  *
  *   @param u The source of of the command
  *                      if this is null we will use the
  *                      servername.
  *
  * @param cmd The command to send
  *                        this takes veradic arguments
  *  @param arg Aguments to the command
  *  @param ...  Varatic arguments for CMD
  *  @return void
  *
  * Note this is perfered to be used because it assumes
  *  a sender and makes proper format :)
  */


void send_cmd(char* source,char *cmd,char *arg, ...)
{
	char	tmp[512];
	va_list	args;

	if (!cmd)
		return;

	va_start(args,arg);
	vsnprintf(tmp,512,arg,args);

	if (source)
		send_line(":%s %s :%s",source,cmd,tmp);
	else
		send_line(":%s %s :%s",CfgSettings.servername,cmd,tmp);

	va_end(args);
	memset(tmp,0,sizeof(tmp));

	return;
}


/********************************************/


void sendto_one (User *source, User *target, char *fmt, ...)
{
	va_list args;
	char    tmp[512];

	if (!target || !source)
		return;

	va_start (args, fmt);
	vsnprintf (tmp, 512, fmt, args);
	va_end (args);

	if (!tmp)
		return;

	source->lastsent	= time(NULL);

	send_line (":%s %s %s :%s", (IRCd.ts6) ? source->uid : source->nick, (CfgSettings.privmsg) ? "PRIVMSG" : "NOTICE",
                HasId(target) ? target->uid : target->nick, tmp);
	return;
}

void sendto_one_privmsg (User *source, User *target, char *fmt, ...)
{
    va_list args;
    char    tmp[512];

    if (!target || !source)
        return;

    va_start (args, fmt);
    vsnprintf (tmp, 512, fmt, args);
    va_end (args);

    if (!tmp)
        return;

    source->lastsent    = time(NULL);

    send_line (":%s PRIVMSG %s :%s", (IRCd.ts6) ? source->uid : source->nick,
                HasId(target) ? target->uid : target->nick, tmp);
    return;
}

void sendto_one_notice (User *source, User *target, char *fmt, ...)
{
    va_list args;
    char    tmp[512];

    if (!target || !source)
        return;

    va_start (args, fmt);
    vsnprintf (tmp, 512, fmt, args);
    va_end (args);

    if (!tmp)
        return;

    source->lastsent    = time(NULL);

    send_line (":%s NOTICE %s :%s", (IRCd.ts6) ? source->uid : source->nick,
                HasId(target) ? target->uid : target->nick, tmp);
    return;
}


void sendto_ircops (User *source, char *fmt, ...)
{
	va_list		args;
	char		tmp[512];
	dlink_node	*dl;
	User		*u;

	if (!source)
		return;

	va_start	(args, fmt);
	vsnprintf	(tmp, 512, fmt, args);
	va_end		(args);

	if (!tmp)
		return;

	DLINK_FOREACH (dl, userlist.head)
	{
		u	= (User *) dl->data;

		if (u->oper)
			sendto_one_notice (source, u, "%s", tmp);
	}

	return;
}

void sendto_channel(User* src,Channel* trg, char* fmt,...)
{
	va_list args;
	char    tmp[512];

	if (!trg || !src)
		return;

	va_start (args, fmt);
	vsnprintf (tmp, 512, fmt, args);
	va_end (args);

	src->lastsent	= time(NULL);

	send_line (":%s PRIVMSG %s :%s", (IRCd.ts6) ? src->uid : src->nick, trg->name, tmp);

	return;
}

void sendto_one_numeric (char *source, User *target, int numeric, char *fmt, ...)
{
    va_list     args;
    char        tmp[512];

    if (!source || !target)
        return;

    va_start    (args, fmt);
    vsnprintf   (tmp, 512, fmt, args);
    va_end      (args);

    send_line (":%s %d %s :%s", (IRCd.ts6) ? Me->sid : Me->name, numeric, HasId(target) ? target->uid : target->nick, tmp);

    return;
}

void sendto_server (User *source, char *fmt, ...)
{
	va_list		args;
	char		tmp[512];

	va_start	(args, fmt);
	vsnprintf	(tmp, 512, fmt, args);
	va_end		(args);

	send_line (":%s %s", (source)? HasId(source)? source->uid : source->nick : CfgSettings.servername, args);

	return;
}

int sendto_socket (Socket *sock, char *fmt, ...)
{
	char	tmp[1024];
	int		n;
	va_list	args;
	dlink_node	*dl;

	if ((!sock) || (sock->sd <= 0))
		return;

	memset (tmp, '\0', sizeof(tmp));

	va_start (args, fmt);
	vsnprintf (tmp, 1024, fmt, args);
	va_end (args);

	if (!strstr (tmp, "\n"))
		strlcat (tmp, "\n", sizeof(tmp));

        /* This should prevent sending from blocking */
    n       = sendq_flush (sock);

    if (n == -1)
    {
            if (errno != EAGAIN)
            {
				close (sock->sd);
				dl = dlink_find_delete (sock, &sockets);
				dlink_free (dl);
				free (sock);
                return 1;
            }
            else
            {
                sendq_add (sock, tmp, strlen(tmp), 0);
                return 0;
            }
    }
    else if (n == 0)
    {
        sendq_add (sock, tmp, strlen(tmp), 0);
        return 0;
    }

    if ((n = write (sock->sd, tmp, strlen(tmp))) == -1)
    {
        if (errno != EAGAIN)
        {
			close (sock->sd);
			dl = dlink_find_delete (sock, &sockets);
			dlink_free (dl);
			free (sock);
            return 1;
        }
        else
        {
			sendq_add (sock, tmp, strlen(tmp), 0);
            return 0;
        }
    }
    sentbytes += n;

    if (n != strlen(tmp))
        sendq_add (sock, tmp, strlen(tmp), n);

    return 0;
}

void uplink_cleanup (void *message, int exitlocal)
{
	int	olduselogchan;

	if (!servsock || (servsock->sd == -1))
		return;

	olduselogchan	= uselogchan;
	uselogchan		= 0;

	exit_remote_users ();

	if (exitlocal)
		exit_local_users (message);

	exit_all_servs ();

	Uplink	= NULL;

	if (Me->uplink)
		Me->uplink	= NULL;

	close (servsock->sd);
	servsock->sd	= -1;
	uselogchan		= olduselogchan;
}

struct hostent *get_hostname(char *para)
{
  char hostname[MAXSERV];
  char *host;
  struct hostent *local_host = NULL;
  struct hostent *host_ptr;

  if (para == NULL)
  {
    gethostname(hostname, MAXSERV - 1);
    hostname[MAXSERV - 1] = '\0';
    host = hostname;
  }
  else
  {
    host = para;
  }
  host_ptr = gethostbyname(host);
  if (host_ptr != NULL)
  {
    local_host = malloc(sizeof(struct hostent));
    memcpy(local_host, host_ptr, sizeof(struct hostent));
  }

  return local_host;
}


/* vim: set tabstop=4 shiftwidth=4 expandtab */


void socket_cleanup()
{
	Socket* s;
	dlink_node *dl, *tdl;
	DLINK_FOREACH_SAFE(dl,tdl,sockets.head)
	{
		s = dl->data;
		if (s->sd > 0)
		{
			sendq_flush(s);
			shutdown(s->sd,2);
		}
		dlink_delete(dl,&sockets);
	}
}
